#include "uogetopt.h"
#include "_stralloc.h"
#include "_error.h"
#include "_str.h"
#include "_ip4.h"
#include "_alloc.h"
#include "_buffer.h"
#include "_scan.h"
#include "fmt.h"
#include "getln.h"
#include "cdb.h"
#include "byte.h"
#include "open.h"
#include "strhash.h"
#include "bailout.h"
#include "accorec.h"
#include "die_write.h"

accorec_t *
accorec_new(strhash_t *hash, const void *name, unsigned int namelen, 
	int multilevel)
{
	accorec_t *m;
	m=(accorec_t *) alloc (sizeof(*m));
	if (!m) oom();
	m->bytes=0;
	if (multilevel) {
		m->others=(strhash_t *) alloc(sizeof(strhash_t));
		if (!m->others) oom();
		if (-1==strhash_create(m->others,1,32,strhash_hash))
			oom();
	} else
		m->others=0;

	if (-1==strhash_enter(hash,1,name,namelen,
		0,(char *)m,sizeof(m))) oom();
	return m;
}
accorec_t *
accorec_lookup(strhash_t *hash, const void *name, unsigned int namelen)
{
	accorec_t *x;
	if (strhash_lookup(hash,name,namelen,(char **)&x,0)) 
		return x;
	return 0;
}
int
accorec_add(accorec_t *rec,
    const void *othername, unsigned int othernamelen,
	unsigned long packets, unsigned long bytes)
{
	uo_uint64_t *x;
	uo_uint64_t y;
	rec->bytes+=bytes;
	(void) packets; /* rec->packets+=packets; */
	if (rec->others) {
		if (strhash_lookup(rec->others,othername,othernamelen,(char **)&x,0)) {
			(*x)+=bytes;
			return 1;
		}
		y=bytes;
		if (-1==strhash_enter(rec->others,1,othername,othernamelen,
			1,(char *)&y,sizeof(y))) 
				return 0;
	}
	return 1;
}

void
accorec_print_bytes(uo_uint64_t b)
{
	char nb[FMT_ULONG];
#define GIG (1000*1000*1000)
#define NINEZEROES "000000000"
	unsigned long low;
	unsigned long high;
	low=b % GIG;
	high=(b-low) / GIG;
	if (high) {
		unsigned int l;
		if (-1==buffer_put(buffer_1,nb,fmt_ulong(nb,high))) die_write();
		l=fmt_ulong(nb,low);
		if (-1==buffer_put(buffer_1,NINEZEROES,9-l)) die_write();
		if (-1==buffer_put(buffer_1,nb,l)) die_write();
	} else
		if (-1==buffer_put(buffer_1,nb,fmt_ulong(nb,b))) die_write();
}

void
accorec_print(char *key, unsigned int keylen, accorec_t *x)
{
	accorec_print_name1(key,keylen);
	if (-1==buffer_puts(buffer_1,": ")) die_write();
	accorec_print_bytes(x->bytes);
	if (-1==buffer_puts(buffer_1,"\n")) die_write();

	if (x->others) {
		strhash_walkstart(x->others);
		while (1) {
			char *k;
			unsigned int kl;
			uo_uint64_t *b;
			int y;
			y=strhash_walk(x->others,&k,&kl, (char **)&b,0);
			if (!y)
				break;
			accorec_print_name1(key,keylen);
			if (-1==buffer_puts(buffer_1,"<->")) die_write();
			accorec_print_name2(k,kl);
			if (-1==buffer_puts(buffer_1,": ")) die_write();
			accorec_print_bytes(*b);
			if (-1==buffer_puts(buffer_1,"\n")) die_write();
			if (-1==buffer_flush(buffer_1)) die_write();
		}
	}
}

void
accorec_print_class(char *x, unsigned int l)
{
	if (-1==buffer_put(buffer_1,x,l))
		xbailout(111,errno,"failed to write",0,0,0);
}


void
accorec_print_ipv4(char *x, unsigned int l)
{
	char s[IP4_FMT];
	(void) l;
	if (-1==buffer_put(buffer_1,s,ip4_fmt(s,x)))
		xbailout(111,errno,"failed to write",0,0,0);
}

