#include "uogetopt.h"
#include "_stralloc.h"
#include "_error.h"
#include "_str.h"
#include "_ip4.h"
#include "_alloc.h"
#include "_buffer.h"
#include "_scan.h"
#include "fmt.h"
#include "getln.h"
#include "byte.h"
#include "open.h"
#include "strhash.h"
#include "bailout.h"
#include "interface.h"
#include "accorec.h"
#include "die_write.h"
#include "cdb.h"

static strhash_t ihash;
static struct cdb cdb;

static void
getclass_finish(stralloc *x)
{
	uint32 dpos = cdb_datapos(&cdb);
	unsigned int len = cdb_datalen(&cdb);
	if (!stralloc_ready(x,len)) oom();
	x->len=len;
	if (-1==cdb_read(&cdb,x->s,x->len,dpos))
		bailout(errno,"failed to read CDB",0,0,0);
}


static void
getclass(stralloc *sa,const char *cip)
{
	char *ip;
	union { char *c; const char *cc; }  u;
	u.cc=cip;
	ip=u.c;
	switch(cdb_find(&cdb,ip,4)) {
		case -1: bailout(errno,"failed to read CDB",0,0,0);
		case 1: getclass_finish(sa); return;
	}
	switch(cdb_find(&cdb,ip,3)) {
		case -1: bailout(errno,"failed to read CDB",0,0,0);
		case 1: getclass_finish(sa); return;
	}
	switch(cdb_find(&cdb,ip,2)) {
		case -1: bailout(errno,"failed to read CDB",0,0,0);
		case 1: getclass_finish(sa); return;
	}
	switch(cdb_find(&cdb,ip,1)) {
		case -1: bailout(errno,"failed to read CDB",0,0,0);
		case 1: getclass_finish(sa); return;
	}
	switch(cdb_find(&cdb,ip,0)) {
		case -1: bailout(errno,"failed to read CDB",0,0,0);
		case 1: getclass_finish(sa); return;
	}
	if (!stralloc_copys(sa,"rest")) oom();
}

void
accorecord(const char *ipfrom,const char *ipto,
			unsigned long packets,unsigned long bytes)
{
	accorec_t *f;
	accorec_t *t;
	static stralloc cfrom;
	static stralloc cto;

	getclass(&cfrom,ipfrom);
	getclass(&cto,ipto);

	f=accorec_lookup(&ihash,ipfrom,4); /* XXX ipv4 */
	if (!f)
		f=accorec_new(&ihash,ipfrom,4,1);
	if (!f)
		oom();
	t=accorec_lookup(&ihash,ipto,4); /* XXX ipv4 */
	if (!t)
		t=accorec_new(&ihash,ipto,4,1);
	if (!t)
		oom();
	if (!accorec_add(t,cfrom.s,cfrom.len,packets,bytes)) 
		oom();
	if (!accorec_add(f,cto.s,cto.len,packets,bytes)) 
		oom();
}


static void dump(void)
{
	strhash_walkstart(&ihash);
	while (1) {
		int x;
		char *k;
		unsigned int kl;
		accorec_t *i;
		x=strhash_walk(&ihash,&k,&kl, (char **)&i,0);
		if (!x)
			break;
		accorec_print(k,kl,i);
	}
}

static uogetopt_t opts[]={
{0,"",UOGO_MINARGS | UOGO_HIDDEN,0, 2 , 0,0},
{0,"",UOGO_MAXARGS | UOGO_HIDDEN,0, 2 , 0,0},
{'\0',0,UOGO_FLAG,0,0,0,0}
};
static const char *usagestring=
"usage: cacco-ip-class [options] CDB <in >out\n";
int 
main(int argc, char **argv)
{
	int fd;
	bailout_progname(argv[0]);
	uogetopt(flag_bailout_log_name,PACKAGE,VERSION,
		&argc,argv,uogetopt_out,
		usagestring, opts,0);
	fd=open_read(argv[1]);
	if (fd==-1) bailout(errno,"failed to open ",argv[1],0,0);
	cdb_init(&cdb,fd);
	accorec_print_name1=accorec_print_ipv4;
	accorec_print_name2=accorec_print_class;

	if (-1==strhash_create(&ihash,256,32,strhash_hash)) oom();
	readloop(0);
	dump();
	if (-1==buffer_flush(buffer_1)) die_write();
	exit(0);
}
